/// Provides helper functions for [Node.js](https://nodejs.org) interoperability.
library nodejs_interop.util;

import 'dart:async';
import 'dart:js_util';
import 'dart:typed_data';
import 'package:js/js.dart';
import 'js.dart';
import 'node.dart';

/// Converts the specified [promise] to a [Future].
Future<T> convertToFuture<T>(Promise<T> promise) {
  var completer = new Completer<T>();
  promise.then(allowInterop(completer.complete), allowInterop(completer.completeError));
  return completer.future;
}

/// Converts the specified [readable] stream to a [Stream].
Stream<T> convertToStream<T>(Readable<T> readable) {
  var controller = new StreamController<T>();
  readable
    ..on('error', allowInterop((error) => controller..addError(error)..close()))
    ..on('data', allowInterop((data) => controller.add(data)))
    ..on('end', allowInterop(() => controller.close()));

  return controller.stream;
}

/// Recursively converts the specified JavaScript [value] to a Dart object.
dynamic dartify(value) {
  if (value == null) return null;
  if (value is bool || value is num || value is DateTime || value is String || value is TypedData) return value;
  if (value is List) return value.map(dartify).toList();
  return new Map<String, dynamic>.fromIterable(JSObject.keys(value), value: (key) => dartify(getProperty(value, key)));
}

/// Requires the JavaScript module with the specified [identifier].
///
/// Throws a [DeferredLoadException] if the module could not be required.
/// If a [onError] handler is provided, it is called with the [identifier] as argument, and its return value is used instead.
///
/// This function is prefered to the [require] function because it properly exposes to the Dart engine the types exported by the required module.
T loadLibrary<T>(String identifier, {T onError(String identifier)}) {
  try {
    var module = require(identifier);
    var moduleId = '\$${identifier.replaceAll(new RegExp('[-/@.]'), '_')}';
    if (!hasProperty(global, moduleId)) setProperty(global, moduleId, module);
    return module;
  }

  on JSError {
    if (onError != null) return onError(identifier);
    throw new DeferredLoadException(identifier);
  }
}
