import 'dart:async';
import 'package:js/js.dart';
import 'package:nodejs_interop/nodejs_interop.dart';
import 'package:test/test.dart';

/// Tests the features of the [Resolver] class.
void main() => group('Resolver', () {
  final net = loadLibrary<NetModule>('net');
  loadLibrary('dns');

  group('.getServers()/.setServers()', () {
    test('should return a non-empty list', () {
      expect(new Resolver().getServers(), isNotEmpty);
    });

    test('should get and set the list of DNS servers', () {
      var servers = const ['127.0.0.1', '::1'];
      var resolver = new Resolver()..setServers(servers);
      expect(resolver.getServers(), orderedEquals(servers));
    });
  });

  group('.resolve4()', () {
    test('should resolve the hostname into a list of IPv4 addresses', () async {
      var completer = new Completer<List>();
      new Resolver().resolve4('belin.io', allowInterop((error, [addresses]) {
        if (error != null) completer.completeError(error);
        else completer.complete(addresses);
      }));

      expect(net.isIPv4((await completer.future).single), isTrue);
    });

    test('should resolve the hostname into a list of IPv4 `AddressRecord` instances', () async {
      var completer = new Completer<List>();
      new Resolver().resolve4('belin.io', new ResolveOptions(ttl: true), allowInterop((error, [records]) {
        if (error != null) completer.completeError(error);
        else completer.complete(records);
      }));

      var record = (await completer.future).single;
      expect(record, const isInstanceOf<AddressRecord>());
      expect(net.isIPv4(record.address), isTrue);
    });
  });

  group('.resolve6()', () {
    test('should resolve the hostname into a list of IPv6 addresses', () async {
      var completer = new Completer<List>();
      new Resolver().resolve6('google.com', allowInterop((error, [addresses]) {
        if (error != null) completer.completeError(error);
        else completer.complete(addresses);
      }));

      expect(net.isIPv6((await completer.future).single), isTrue);
    });

    test('should resolve the hostname into a list of IPv6 `AddressRecord` instances', () async {
      var completer = new Completer<List>();
      new Resolver().resolve6('google.com', new ResolveOptions(ttl: true), allowInterop((error, [records]) {
        if (error != null) completer.completeError(error);
        else completer.complete(records);
      }));

      var record = (await completer.future).single;
      expect(record, const isInstanceOf<AddressRecord>());
      expect(net.isIPv6(record.address), isTrue);
    });
  });

  group('.resolveAny()', () {
    test('should resolve the hostname into a list of `DnsRecord` instances', () async {
      var completer = new Completer<List<DnsRecord>>();
      new Resolver().resolveAny('belin.io', allowInterop((error, [records]) {
        if (error != null) completer.completeError(error);
        else completer.complete(records);
      }));

      expect(await completer.future, isNotEmpty);
    });
  });

  group('.resolveCname()', () {
    test('should resolve the hostname into a list of canonical names', () async {
      var completer = new Completer<List<String>>();
      new Resolver().resolveCname('www.belin.io', allowInterop((error, [records]) {
        if (error != null) completer.completeError(error);
        else completer.complete(records);
      }));

      expect(await completer.future, orderedEquals(['ns.belin.io']));
    });
  });

  group('.resolveMx()', () {
    test('should resolve the hostname into a list of mail exchange records', () async {
      var completer = new Completer<List<MXRecord>>();
      new Resolver().resolveMx('google.com', allowInterop((error, [records]) {
        if (error != null) completer.completeError(error);
        else completer.complete(records);
      }));

      expect(await completer.future, isNotEmpty);
    });
  });

  group('.resolveNs()', () {
    test('should resolve the hostname into a list of name servers', () async {
      var completer = new Completer<List<String>>();
      new Resolver().resolveNs('belin.io', allowInterop((error, [servers]) {
        if (error != null) completer.completeError(error);
        else completer.complete(servers);
      }));

      expect(await completer.future, unorderedEquals(['ns-39-c.gandi.net', 'ns-68-b.gandi.net', 'ns-230-a.gandi.net']));
    });
  });

  group('.resolveSoa()', () {
    test('should resolve the hostname into a start of authority record', () async {
      var completer = new Completer<SoaRecord>();
      new Resolver().resolveSoa('belin.io', allowInterop((error, [record]) {
        if (error != null) completer.completeError(error);
        else completer.complete(record);
      }));

      var record = await completer.future;
      expect(record.nsname, equals('ns1.gandi.net'));
      expect(record.hostmaster, equals('hostmaster.gandi.net'));
    });
  });

  group('.resolveTxt()', () {
    test('should resolve the hostname into a list of text queries', () async {
      var completer = new Completer<List<List<String>>>();
      new Resolver().resolveTxt('belin.io', allowInterop((error, [records]) {
        if (error != null) completer.completeError(error);
        else completer.complete(records);
      }));

      var records = [];
      for (var record in await completer.future) record.forEach(records.add);
      expect(records, unorderedEquals(['firebase=cb-belin', 'google-site-verification=4Lpj0fzF1aePRZ_Zj3SOSn0Gv24ztXPxZ_lkTP1ouFQ']));
    });
  });

  group('.reverse()', () {
    test('should resolve the address into a list of hostnames', () async {
      var completer = new Completer<List<String>>();
      new Resolver().reverse('192.30.253.113', allowInterop((error, [hostnames]) {
        if (error != null) completer.completeError(error);
        else completer.complete(hostnames);
      }));

      expect(await completer.future, orderedEquals(['lb-192-30-253-113-iad.github.com']));
    });
  });
});
